/*   Guido Montorsi, 26/1/1999
 simulation program for Lucent contract 
 PCCC rate 1/3 
 constituent CC: 1/2 + 1/1  8-state / 8-state
 information frame : 320-640-5120 
 interleaver length: 320-640-5120 
 AWGN and Fading Channels

*/  

/*************************************************************************/

#define vel 30       /* Speed (km/h) */

/* 320 
#define fs              64E3    
#define SYMBOL_PER_SLOT	40
#define SLOT_PER_FRAME	16
#define nframe		320       
#define ninter		320       
#define ncod		972 */     


/* 640 */
#define fs              128E3
#define SYMBOL_PER_SLOT 80
#define SLOT_PER_FRAME  16
#define nframe	  640       
#define ninter	  640       
#define ncod		  1932      


/* 5120 
#define fs              128E3
#define SYMBOL_PER_SLOT 80
#define SLOT_PER_FRAME  128
#define nframe	  5120       
#define ninter	  5120       
#define ncod		15372 */  

#define stat_max        10

/* include */
#include <stdio.h>
#ifdef WIN32
#include <conio.h> /* For _kbhit */
#endif
#include <math.h>
#include <time.h>
#include <stdlib.h>
#include "../basic_demod_types.h"
#include "../lee3c.h"


/* Definition of functions */
/********************************************************************/ 
/* Global variables for coding */

int frame[2][nframe];                  /*   information bits  */
int lower[nframe];                   

int codeword[ncod];                    /*   encoded bits   */
int codeword1[ninter*2+6];               /*   encoded bits   */
int codeword2[ninter+6];                 /*   encoded bits   */

double s[ncod];                        /*   received signals (with AWGN)     */

int    s_c[ncod];                      /*   quantized received signals (with AWGN)     */
int    s_c1[ninter*2+6];                      /*   quantized received signals (with AWGN)     */
int    s_c2[ninter+6];                      /*   quantized received signals (with AWGN)     */

int frame_dec[nframe];                 /*   decoded information bits    */

int perm[ninter];                        /* permutation */
int verbose=1;
char stringa[40];                      /* dummy char string */

FILE *f_input,*f_perm;     /* files: input, permutation, and output */
FILE *f_stat,*f_BER,*f_FER; 
unsigned long int l;            /* LFSR circuit (32 taps) period= 2^32 - 1*/
unsigned long int l_start;      /* valore iniziale LFSR */
unsigned int seeds[2];

long int vc_un;           /* variabile intera 31 bit per    */
                          /*   l'algoritmo delle congruenze */
long int vc_un_start;     /* valore iniziale */

double s_n_db;             /* rapporto segnale/disturbo e varianza */
double s_n,var2,var,var3;
 
clock_t ck,ck_t;          /* calcolo execution time */
int numero_step;          /* numero di simulazioni da eseguire */
double delta_s_n_db;       /* incremento per ogni simulazione */
int step;
int numero_frame;               /* numero di frame da simulare*/
int f;                         /* frame corrente */
int niterazioni,ncheck,nerr_max;
int f_delay;
int bit_err[20],frame_err[20]; 
int stat_err[20][stat_max];
                            /* bit di informazione errati, 
                               frame errati*/


int i,i0,i1,i2,i3,j,bit;
int curr_iter;
int i;
int		  siso_output[nframe];
static int llri1[ninter];
static int llri2[ninter];
int	nnew;
const int nsteps=ninter+3;

/* Global variables for channel */
static int     symbols_per_slot=SYMBOL_PER_SLOT;
static int     Rake_fingers=4;
static int     pc_flag=0;
static int     count=0;
int            bit_per_slot=2*SYMBOL_PER_SLOT;
struct NOISE   noise_signal [SYMBOL_PER_SLOT];
struct CHANNEL ch_response  [SYMBOL_PER_SLOT];
struct DATA    slot   [SLOT_PER_FRAME][SYMBOL_PER_SLOT];
struct DATA    soft_output[SLOT_PER_FRAME][SYMBOL_PER_SLOT];
static double	ave_signal_pwr_per_slot[SLOT_PER_FRAME];
static double	ave_noise_pwr_per_slot[SLOT_PER_FRAME];
static double   level;
static double   total_rx_pwr;
static double   total_rx_nse;

#include "../initialize.c"
#include "../starting_step.c"
#include "encoder11_8.c"
#include "encoder12_8.c"
#include "../lee3c.c"
#include "../basic_demod.c"
#include "../channel.c"
#include "../eopq.c"
#include "siso11_8_inner.c"
#include "siso12_8_inner.c"
#include "../scrambler-descrambler.c"
#include "../ber_meter.c"
#include "../results.c"


/*************************************************************************/
void main()              

{    /* main */
		char tmpbuf[128];

		sprintf(tmpbuf,"Data/input%d-%d.txt",vel,nframe);
		f_input=fopen(tmpbuf,"r"); /* read-only input file */

		sprintf(tmpbuf,"Data/FER%d-%d.txt",vel,nframe);
		f_FER     =fopen(tmpbuf, "a" );  /* FER file */

		sprintf(tmpbuf,"Data/BER%d-%d.txt",vel,nframe);
		f_BER     =fopen(tmpbuf, "a" );  /* BER file */

		sprintf(tmpbuf,"Data/statistic%d-%d.txt",vel,nframe);
		f_stat    =fopen(tmpbuf, "a" );        /* Statistic file */

#ifdef WIN32
		_strdate( tmpbuf );
#endif
		fprintf(f_BER, "\n# %s",tmpbuf);
		fprintf(f_FER, "\n# %s",tmpbuf);
		fprintf(f_stat,"\n# %s",tmpbuf );

#ifdef WIN32
		_strtime( tmpbuf );
#endif
		fprintf(f_BER, " %s",tmpbuf );
		fprintf(f_FER, " %s",tmpbuf );
		fprintf(f_stat," %s",tmpbuf );

		printf("PCCC (8 8) Rate 1/3, N=%d, v=%d km/h, symbol rate=%.2e \n",ninter,vel,fs);
		fprintf(f_BER, " - PCCC (8 8) Rate 1/3, N=%d, v=%d km/h, symbol rate=%.2e \n",ninter,vel,fs);
	   fprintf(f_FER, " - PCCC (8 8) Rate 1/3, N=%d, v=%d km/h, symbol rate=%.2e \n",ninter,vel,fs);
	   fprintf(f_stat," - PCCC (8 8) Rate 1/3, N=%d, v=%d km/h, symbol rate=%.2e \n",ninter,vel,fs);

/* Initialization of simulation */
      initialize();
		
/* Get R[][], S[][], O[][] and scale ChannelTap_dB[] to linear scale ChannelTap[] */
	   load_lee();

	   
/***********************************************************************/

for (step=0;step<numero_step;step++)

{    /* Eb/N0 loop */

starting_step();

count = 0;
level = (double)0.7071067812;
total_rx_pwr=0.;
total_rx_nse=0.;
nnew=0;

ck=clock();
for (f=0;f<numero_frame;f++)
{
 /* Frame loop */
	  
	seeds[0]=l;
	seeds[1]=vc_un;

	for (i=0;i<nframe;i++)frame[nnew][i]=lfsr();

	Encoder12_8(nsteps,frame[nnew],codeword1);
	for (i=0;i<ninter;i++) lower[i]=frame[nnew][perm[i]];
	Encoder11_8(nsteps,lower,codeword2);

/* The bits are reordered as follows: 
1st bit upper encoder
2nd bit upper encoder
1st bit lower encoder
...
6 terminating bits of upper encoder
6 terminating bits of lower encoder 
*/
	for(i=0;i<ninter;i++)
	{
		codeword[3*i]   = codeword1[2*i];
		codeword[3*i+1] = codeword1[2*i+1];
		codeword[3*i+2] = codeword2[i];
	}
	for(i=0;i<6;i++)
	{
	codeword[3*ninter+i]   = codeword1[2*ninter+i];
	codeword[3*ninter+6+i] = codeword2[ninter+i];
	}


/*Phase 2 Channel */

	channel_fading();


/* Reorder LLR */
	for(i=0;i<ninter;i++)
	{
		s_c1[2*i] = s_c[3*i] ;
		s_c1[2*i+1]=s_c[3*i+1];
		s_c2[i]    =s_c[3*i+2];
	}

	for(i=0;i<6;i++)
	{
	s_c1[2*ninter+i] = s_c[3*ninter+i];
	s_c2[ninter+i]   = s_c[3*ninter+6+i];
	}


	if (f>=f_delay){

/* Reset extrinsic information */
	for(i=0;i<ninter;i++)llri1[i]=0;

/* Iteration loop for the decoder */
	for(curr_iter=0;curr_iter < niterazioni;curr_iter++)
	{
		SISO12_8_Inner(nsteps,llri1,s_c1);
		scrambler(perm,llri1,llri2);
		SISO11_8_Inner(nsteps,llri2,s_c2);
		descrambler_dec(perm,llri2,llri1,siso_output);
		BER_meter();
	}

	if (frame_err[niterazioni-1] >= nerr_max && (f > 1000)){f++; break;}

#ifdef WIN32
	/* Stop simulation if 'q' hitted (NOT standard) */
	if (_kbhit())if(_getch()=='q'){f++;break;}
#endif
   if (f%ncheck==ncheck-1)
	{
		ck_t=clock();

		intermediate_results();
		ck=ck_t;
	}
	}
	nnew ^= 1;
} /* per tutti i frame */

final_results();

}  /* per tutti gli step */
fclose(f_stat);
fclose(f_FER);
fclose(f_BER);
}  /* main */ 


