/*   Guido Montorsi 12/1/1999
 simulation program for Lucent contract 
 first version: PHASE 1
 SCCC rate 1/3 
 constituent CC: 2/3 + 1/2 punctured 4-state / 4-state
 information frame:  320-640-5120 
 interleaver length: 483-963-7683
 Mobile Channel + Basic demod
*/  

/*************************************************************************/
#define vel 3      /* Speed */

/*320
#define fs              64E3
#define SYMBOL_PER_SLOT 40
#define SLOT_PER_FRAME  16
#define nframe	  320    
#define ninter	  483    
#define ncod		  970  */  

/* 640 */
#define fs              128E3
#define SYMBOL_PER_SLOT 80
#define SLOT_PER_FRAME  16
#define nframe	  640    
#define ninter	  963    
#define ncod		  1930     

/* 5120 
#define fs              128E3
#define SYMBOL_PER_SLOT 80
#define SLOT_PER_FRAME  128
#define nframe		5120    
#define ninter		7683    
#define ncod		15370  */
   
#define stat_max        10

/* include header */
#include <stdio.h>
#ifdef WIN32 
#include <conio.h> 
#endif
#include <math.h>
#include <time.h>
#include <stdlib.h>
#include "../basic_demod_types.h"
#include "../lee3c.h"

/********************************************************************/ 
/* Global variables */

const int nstepsO=nframe+2;
const int nstepsI=ninter+2;

int frame[2][nframe];                     /*   information bits  */
int outer[ninter];                       /*   output of the outer encoder */
int inner[ninter];                       /*   input  of the inner encoder */
int codeword[ncod];                    /*   encoded bits   */
double s[ncod];                        /*   received signals (with AWGN)     */
int    s_c[ncod];                      /*   quantized received signals (with AWGN)     */
int frame_dec[nframe];                 /*   decoded information bits    */
int perm[ninter];                        /* permutation */
int verbose=1;
char stringa[40];                      /* dummy char string */

FILE *f_input,*f_perm;     /* files: input, permutation, and output */
FILE *f_stat,*f_BER,*f_FER; 
unsigned long int l;            /* LFSR circuit (32 taps) period= 2^32 - 1*/
unsigned long int l_start;      /* valore iniziale LFSR */
unsigned int seeds[2];

long int vc_un;           /* variabile intera 31 bit per    */
                          /*   l'algoritmo delle congruenze */
long int vc_un_start;     /* valore iniziale */

double s_n_db;             /* rapporto segnale/disturbo e varianza */
double s_n,var2,var;
 
clock_t ck,ck_t;          /* calcolo execution time */
int numero_step;          /* numero di simulazioni da eseguire */
double delta_s_n_db;       /* incremento per ogni simulazione */
int step;
int fading;
int numero_frame;               /* numero di frame da simulare*/
int f;                         /* frame corrente */
int niterazioni,ncheck,nerr_max;

int bit_err[20],frame_err[20]; 
int stat_err[20][stat_max];
                            /* bit di informazione errati, 
                               frame errati*/


int i,i0,i1,i2,i3,j,bit;
int curr_iter;
int i;
int		  siso_output[nframe];
static int llri1[ninter];
static int llro2[ninter];
int	nnew;

/* Global variables for channel */
static int     symbols_per_slot=SYMBOL_PER_SLOT;
static int     Rake_fingers=4;
static int     pc_flag=0;
static int     count=0;
struct NOISE   noise_signal[SYMBOL_PER_SLOT];
struct CHANNEL ch_response [SYMBOL_PER_SLOT];
struct DATA    slot        [SLOT_PER_FRAME][SYMBOL_PER_SLOT];
struct DATA    soft_output [SLOT_PER_FRAME][SYMBOL_PER_SLOT];
static double	ave_signal_pwr_per_slot[SLOT_PER_FRAME];
static double	ave_noise_pwr_per_slot [SLOT_PER_FRAME];
static double   level;
static double   total_rx_pwr;
static double   total_rx_nse;
int f_delay;

#include "../initialize.c"
#include "../starting_step.c"
#include "encoder23_4_pun.c"
#include "encoder12_4.c"
#include "../lee3c.c"
#include "../basic_demod.c"
#include "../channel.c"
#include "../eopq.c"
#include "siso12_4_inner.c"
#include "siso23_4_pun_outer.c"
#include "../scrambler-descrambler.c"
#include "../ber_meter.c"
#include "../results.c"

/*************************************************************************/
void main()              

{    /* main */
		char tmpbuf[128];

		sprintf(tmpbuf,"Data/input%d-%d.txt",vel,nframe);
		f_input   =fopen(tmpbuf,"r");  /* read-only input file */

		sprintf(tmpbuf,"Data/FER%d-%d.txt",vel,nframe);
		f_FER     =fopen(tmpbuf, "a" );  /* FER file */

		sprintf(tmpbuf,"Data/BER%d-%d.txt",vel,nframe);
		f_BER     =fopen(tmpbuf, "a" );  /* BER file */

		sprintf(tmpbuf,"Data/statistic%d-%d.txt",vel,nframe);
		f_stat    =fopen(tmpbuf, "a" );        /* Statistic file */

#ifdef WIN32 
		_strdate( tmpbuf );
#endif
		fprintf(f_BER, "\n# %s",tmpbuf);
		fprintf(f_FER, "\n# %s",tmpbuf);
		fprintf(f_stat,"\n# %s",tmpbuf );

#ifdef WIN32 
		_strtime( tmpbuf );
#endif
		fprintf(f_BER, " %s",tmpbuf );
		fprintf(f_FER, " %s",tmpbuf );
		fprintf(f_stat," %s",tmpbuf );
		printf("SCCC (2/3 4 pun) - (1/2 4), N=%d,v=%d km/h, symbol rate=%.2e \n",nframe,vel,fs);
		fprintf(f_BER, " - SCCC (2/3 4 pun) - (1/2 4), N=%d, v=%d km/h, symbol rate=%.2e \n",nframe,vel,fs);
	   fprintf(f_FER, " - SCCC (2/3 4 pun) - (1/2 4), N=%d, v=%d km/h, symbol rate=%.2e \n",nframe,vel,fs);
	   fprintf(f_stat," - SCCC (2/3 4 pun) - (1/2 4), N=%d, v=%d km/h, symbol rate=%.2e \n",nframe,vel,fs);

/* Initialization */
      initialize();
/* Get R[][], S[][], O[][] and scale ChannelTap_dB[] to linear scale ChannelTap[] */
	   load_lee();

		/* Output data */

	   
/***********************************************************************/

for (step=0;step<numero_step;step++)

{    /* per tutti gli step */

starting_step();
count = 0;
level = (double)0.7071067812;
total_rx_pwr=0.;
total_rx_nse=0.;
nnew=0;


ck=clock();
for (f=0;f<numero_frame;f++)
{
 /* per tutti i frame */
	  
	seeds[0]=l;
	seeds[1]=vc_un;

	for (i=0;i<nframe;i++) frame[nnew][i]=lfsr();

	Encoder23_4_pun(nstepsO,frame[nnew],outer);
	for (i=0;i<ninter;i++) inner[i]=outer[perm[i]];
	Encoder12_4(nstepsI,inner,codeword);


/* Channel */
	channel_fading();
 


	if (f>=f_delay){
	/* Reset extrinsic information */
	for(i=0;i<ninter;i++)llri1[i]=0;

	/* Build buffer for SISO */

	for(curr_iter=0;curr_iter < niterazioni;curr_iter++)
	{
		SISO12_4_Inner(nstepsI,llri1,s_c);
		descrambler(perm,llri1,llro2);
		SISO23_4_pun_Outer(nstepsO,siso_output,llro2);
		scrambler(perm,llro2,llri1);
		BER_meter();
	}

	if (frame_err[niterazioni-1] >= nerr_max && f>1000){f++; break;}
	/* Stop simulation if 'q' hitted (NOT standard) */
#ifdef WIN32
	if (_kbhit())if(_getch()=='q'){f++;break;}
#endif

   if (f%ncheck==ncheck-1)
	{
		ck_t=clock();
		intermediate_results();
		ck=ck_t;
	}
	}
	nnew ^= 1;

} /* per tutti i frame */

final_results();

}  /* per tutti gli step */
fclose(f_stat);
fclose(f_FER);
fclose(f_BER);
}  /* main */ 



